package com.company.testaddon.test_support;

import com.google.common.base.Strings;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.router.RouteConfiguration;
import com.vaadin.flow.router.RouterLayout;
import io.jmix.flowui.testassist.JmixUiTestExtension;
import io.jmix.flowui.view.DefaultMainViewParent;
import io.jmix.flowui.view.View;
import io.jmix.flowui.view.ViewInfo;
import io.jmix.flowui.view.ViewRegistry;
import org.apache.commons.collections4.CollectionUtils;
import org.junit.jupiter.api.extension.ExtensionContext;
import org.springframework.context.ApplicationContext;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class AppUiTestAssist extends JmixUiTestExtension {

    protected void registerViewRoutes(List<String> viewBasePackages, ExtensionContext context) {
        if (CollectionUtils.isEmpty(viewBasePackages)) {
            return;
        }

        ApplicationContext applicationContext = getApplicationContext(context);

        List<ViewInfo> viewInfos = applicationContext.getBean(ViewRegistry.class)
                .getViewInfos().stream()
                .filter(info -> isClassInPackages(info.getControllerClass().getPackageName(), viewBasePackages))
                .collect(Collectors.toList());

        for (ViewInfo view : viewInfos) {
            Class<? extends View> controllerClass = view.getControllerClass();
            Route route = controllerClass.getAnnotation(Route.class);
            if (route == null) {
                continue;
            }

            RouteConfiguration routeConfiguration = RouteConfiguration.forSessionScope();
            if (Strings.isNullOrEmpty(route.value())) {
                continue;
            }

            // Provides ability to overriding Views like in ViewRegistry#registerRoute()
            if (routeConfiguration.isPathAvailable(route.value())) {
                routeConfiguration.removeRoute(route.value());
            }

            if (route.layout() == UI.class) {
                routeConfiguration.setRoute(route.value(), controllerClass);
            } else {
                routeConfiguration.setRoute(route.value(), controllerClass,
                        getParentChain(route, Collections.emptyList()));
            }
        }
    }

    protected List<Class<? extends RouterLayout>> getParentChain(Route route,
                                                                 List<Class<? extends RouterLayout>> defaultChain) {
        Class<? extends RouterLayout> layout = route.layout();
        if (DefaultMainViewParent.class.isAssignableFrom(layout)) {
            return defaultChain;
        }

        // UI is the default route parent, so no need to add it explicitly
        return UI.class == layout ? Collections.emptyList() : List.of(layout);
    }
}
